( function( $, _, Backbone, api, settings ) {

	var happyForms;
	var classes = {};
	classes.models = {};
	classes.collections = {};
	classes.views = {};
	classes.routers = {};

	classes.models.Form = Backbone.Model.extend( {
		idAttribute: 'ID',
		defaults: settings.form,

		getPreviewUrl: function() {
			var previewUrl =
				settings.baseUrl +
				'?post_type=' + this.get( 'post_type' ) +
				'&p=' + this.id +
				'&preview=true';

			return previewUrl;
		},

		isNew: function() {
			return '0' === this.id;
		},

		initialize: function( attrs, options ) {
			Backbone.Model.prototype.initialize.apply( this, arguments );

			this.attributes.parts = new classes.collections.Parts( this.get( 'parts' ), options );
		},

		toJSON: function() {
			var json = Backbone.Model.prototype.toJSON.apply( this, arguments );
			json.parts = json.parts.toJSON();

			return json;
		},

		save: function( options ) {
			var self = this;
			options = options || {};

			var request = wp.ajax.post( 'happyforms-update-form', _.extend( {
				'happyforms-nonce': api.settings.nonce.happyforms,
				happyforms: 1,
				form_id: this.id,
				wp_customize: 'on',
			}, { form: this.toJSON() } ) );

			request.done( function( response ) {
				self.set( response.data );

				if ( happyForms.previewLoaded ) {
					api.previewer.refresh();
				}

				if ( options.success ) {
					options.success( response );
				}
			} );

			request.fail( function( response ) {
				// noop
			} );
		}
	} );

	classes.models.Part = Backbone.Model.extend( {
		initialize: function( attributes ) {
			Backbone.Model.prototype.initialize.apply( this, arguments );

			if ( ! this.id ) {
				var id = happyForms.utils.uniqueId( this.get( 'type' ) + '_', this.collection );
				this.set( 'id', id );
			}
		},

		fetchHtml: function( success ) {
			var data = {
				action: 'happyforms-form-part-add',
				'happyforms-nonce': api.settings.nonce.happyforms,
				happyforms: 1,
				wp_customize: 'on',
				form_id: happyForms.form.id,
				part: this.toJSON(),
			};

			var request = $.ajax( ajaxurl, {
				type: 'post',
				dataType: 'html',
				data: data
			} );

			api.previewer.send( 'happyforms-form-part-disable', {
				id: this.get( 'id' ),
			} );

			request.done( success );
		}
	} );

	classes.collections.Parts = Backbone.Collection.extend( {
		model: function( attrs, options ) {
			var model = PartFactory.model( attrs, options );
			return model;
		}
	} );

	var PartFactory = {
		model: function( attrs, options, BaseClass ) {
			BaseClass = BaseClass || classes.models.Part;
			return new BaseClass( attrs, options );
		},

		view: function( attrs, BaseClass ) {
			BaseClass = BaseClass || classes.views.Part;
			return new BaseClass( attrs );
		},
	};

	HappyForms = Backbone.Router.extend( {
		routes: {
			'build': 'build',
			'setup': 'setup',
			'style': 'style',
		},

		steps: [ 'build', 'setup', 'style' ],
		currentRoute: 'setup',
		form: false,
		previewLoaded: false,
		buffer: [],

		initialize: function( options ) {
			Backbone.Router.prototype.initialize( this, arguments );

			this.listenTo( this, 'route', this.onRoute );
		},

		start: function( options ) {
			this.parts = new Backbone.Collection();
			this.parts.reset( _( settings.formParts ).values() );
			this.form = new classes.models.Form( settings.form, { silent: true } );
			this.actions = new classes.views.Actions( { model: this.form } ).render();
			this.sidebar = new classes.views.Sidebar( { model: this.form } ).render();

			Backbone.history.start();
			api.previewer.previewUrl( this.form.getPreviewUrl() );
		},

		flushBuffer: function() {
			if ( this.buffer.length > 0 ) {
				_.each( this.buffer, function( entry ) {
					api.previewer.send( entry.event, entry.data );
				} );

				this.buffer = [];
			}
		},

		onRoute: function( segment ) {
			this.sidebar.steps.disable();

			var previousStepIndex = this.steps.indexOf( this.currentRoute ) + 1;
			var stepIndex = this.steps.indexOf( segment ) + 1;
			var direction = previousStepIndex < stepIndex ? 1: -1;
			var stepProgress = Math.round( stepIndex / ( this.steps.length ) * 100 );
			var childView;

			switch( segment ) {
				case 'setup':
					childView = new classes.views.FormSetup( { model: this.form } );
					break;
				case 'build':
					childView = new classes.views.FormBuild( { model: this.form } );
					break;
				case 'style':
					childView = new classes.views.FormStyle( { model: this.form } );
					break;
			}

			this.currentRoute = segment;

			this.sidebar.doStep( {
				step: {
					slug: segment,
					index: stepIndex,
					progress: stepProgress,
					count: this.steps.length,
				},
				direction: direction,
				child: childView,
			} );
		},

		forward: function() {
			var nextStepIndex = this.steps.indexOf( this.currentRoute ) + 1;
			nextStepIndex = Math.min( nextStepIndex, this.steps.length - 1 );
			var nextStep = this.steps[nextStepIndex];

			this.navigate( nextStep, { trigger: true } );
		},

		back: function() {
			var previousStepIndex = this.steps.indexOf( this.currentRoute ) - 1;
			previousStepIndex = Math.max( previousStepIndex, 0 );

			var previousStep = this.steps[previousStepIndex];
			this.navigate( previousStep, { trigger: true } );
		},

		setup: function() {
			// noop
		},

		build: function() {
			// noop
		},

		style: function() {
			// noop
		},

	} );

	classes.views.Base = Backbone.View.extend( {
		initialize: function() {
			if ( this.template ) {
				this.template = _.template( $( this.template ).text() );
			}

			this.listenTo( this, 'ready', this.ready );

			// Capture and mute link clicks to avoid
			// hijacking Backbone router and breaking
			// Customizer navigation.
			this.delegate( 'click', '.happyforms-stack-view a', this.muteLink );
		},

		ready: function() {
			// Noop
		},

		muteLink: function( e ) {
			e.preventDefault();
		},

		unbindEvents: function() {
			// Unbind any listenTo handlers
			this.stopListening();
			// Unbind any delegated DOM handlers
			this.undelegateEvents()
			// Unbind any direct view handlers
			this.off();
		},

		remove: function() {
			this.unbindEvents();
			Backbone.View.prototype.remove.apply( this, arguments );
		},
	} );

	classes.views.Actions = classes.views.Base.extend( {
		el: '#customize-header-actions',
		template: '#happyforms-customize-header-actions',

		events: {
			'click #happyforms-save-button': 'onSaveClick',
			'click #happyforms-close-link': 'onCloseClick',
			'onbeforeunload': 'onWindowClose',
		},

		initialize: function() {
			classes.views.Base.prototype.initialize.apply( this, arguments );

			this.listenTo( this.model, 'change', this.onFormChange );
			$( window ).bind( 'beforeunload', this.onWindowClose.bind( this ) );
		},

		render: function() {
			this.$el.html( this.template( {} ) );

			if ( this.model.isNew() ) {
				this.enableSave();
			}

			return this;
		},

		enableSave: function() {
			$( '#happyforms-save-button', this.$el ).removeAttr( 'disabled' );
		},

		disableSave: function() {
			$( '#happyforms-save-button', this.$el ).attr( 'disabled', 'disabled' );
		},

		isDirty: function() {
			return $( '#happyforms-save-button', this.$el ).is( ':enabled' );
		},

		onFormChange: function() {
			this.enableSave();
		},

		onCloseClick: function( e ) {
			if ( this.isDirty() ) {
				var message = $( e.currentTarget ).data( 'message' );

				if ( ! confirm( message ) ) {
					e.preventDefault();
					e.stopPropagation();

					return false;
				} else {
					$( window ).unbind( 'beforeunload' );
				}
			}
		},

		onWindowClose: function() {
			if ( this.isDirty() ) {
				return '';
			}
		},

		onSaveClick: function( e ) {
			e.preventDefault();

			var self = this;

			this.disableSave();

			this.model.save( {
				success: function() {
					window.location.href = $( '#happyforms-close-link' ).attr( 'href' );
				}
			} );
		},
	} );

	classes.views.Sidebar = classes.views.Base.extend( {
		el: '.wp-full-overlay-sidebar-content',

		steps: null,
		current: null,
		previous: null,

		render: function( options ) {
			this.$el.empty();
			this.steps = new classes.views.Steps( { model: this.model } );

			return this;
		},

		doStep: function( options ) {
			var child = options.child.render();
			this.$el.append( child.$el );
			child.trigger( 'ready' );

			if ( this.current ) {
				this.previous = this.current;
				this.current = child;

				this.current.$el.css( 'left', options.direction > 0 ? '300px': '-300px' );
				this.current.$el.animate( {
					left: '0px',
				}, 200 );

				this.previous.$el.animate( {
					left: options.direction > 0 ? '-300px': '300px',
				}, 200, $.proxy( this.onStepComplete, this, options ) );
			} else {
				this.current = child;
				this.current.$el.css( 'left', '0px' );
				this.steps.render( options );
			}
		},

		onStepComplete: function( options ) {
			this.previous.remove();
			$( '.wp-full-overlay-sidebar-content' ).scrollTop( 0 );
			this.steps.render( options );
		}
	} );

	classes.views.Steps = classes.views.Base.extend( {
		el: '#customize-footer-actions',
		template: '#happyforms-form-steps-template',

		events: {
			'click .happyforms-step-previous': 'onPreviousStepClick',
			'click .happyforms-step-next': 'onNextStepClick',
			'click .happyforms-step-save': 'onSaveStepClick',
		},

		render: function( options ) {
			var data = _.extend( {}, options, { form: this.model.toJSON() } );
			this.$el.html( this.template( data ) );
			this.$el.show();
		},

		enable: function() {
			$( 'button', this.$el ).removeAttr( 'disabled' );
		},

		disable: function() {
			$( 'button', this.$el ).attr( 'disabled', 'disabled' );
		},

		onNextStepClick: function( e ) {
			e.preventDefault();
			happyForms.forward();
		},

		onPreviousStepClick: function( e ) {
			e.preventDefault();
			happyForms.back();
		},

		onSaveStepClick: function( e ) {
			e.preventDefault();

			this.model.save( {
				success: function() {
					happyForms.actions.disableSave();
					window.location.href = $( '#happyforms-close-link' ).attr( 'href' );
				}
			} );
		},

	} );

	classes.views.FormBuild = classes.views.Base.extend( {
		template: '#happyforms-form-build-template',

		events: {
			'keyup #happyforms-form-name': 'onNameChange',
			'change #happyforms-form-name': 'onNameChange',
			'click .expand-collapse-all': 'onExpandCollapseAllClick'
		},

		drawer: null,

		initialize: function() {
			classes.views.Base.prototype.initialize.apply( this, arguments );

			this.partViews = new Backbone.Collection();

			this.listenTo( happyForms, 'part-add', this.onPartAdd );
			this.listenTo( happyForms, 'part-duplicate', this.onPartDuplicate );
			this.listenTo( this.model.get( 'parts' ), 'add', this.onPartModelAdd );
			this.listenTo( this.model.get( 'parts' ), 'remove', this.onPartModelRemove );
			this.listenTo( this.model.get( 'parts' ), 'change', this.onPartModelChange );
			this.listenTo( this.model.get( 'parts' ), 'reset', this.onPartModelsSorted );
			this.listenTo( this.model.get( 'parts' ), 'widget-toggle', this.onWidgetToggle );
			this.listenTo( this.partViews, 'add', this.onPartViewAdd );
			this.listenTo( this.partViews, 'remove', this.onPartViewRemove );
			this.listenTo( this.partViews, 'reset', this.onPartViewsSorted );
			this.listenTo( this.partViews, 'add remove reset', this.onPartViewsChanged );
			this.listenTo( this, 'sort-stop', this.onPartSortStop );
		},

		render: function() {
			this.setElement( this.template( this.model.toJSON() ) );
			return this;
		},

		onWidgetToggle: function() {
			var $expandCollapseButton = $( '.expand-collapse-all', this.$el );

			if ( 0 === $( '.happyforms-widget-expanded', this.$el ).length ) {
				$expandCollapseButton.text($expandCollapseButton.data('expand-text')).removeClass('collapse').addClass('expand');
			}

			if ( 0 === $( '.happyforms-widget:not(.happyforms-widget-expanded)', this.$el ).length ) {
				$expandCollapseButton.text($expandCollapseButton.data('collapse-text')).removeClass('expand').addClass('collapse');
			}
		},

		ready: function() {
			this.model.get( 'parts' ).each( function( partModel ) {
				this.addViewPart( partModel );
			}, this );

			$( '.happyforms-form-widgets', this.$el ).sortable( {
				handle: '.happyforms-part-widget-top',

				stop: function ( e, ui ) {
					this.trigger( 'sort-stop', e, ui );
				}.bind( this ),
			} );

			this.drawer = new classes.views.PartsDrawer();
			$( '.wp-full-overlay' ).append( this.drawer.render().$el );
			$( 'body' ).addClass( 'adding-happyforms-parts' );

			this.drawer.$el.animate( {
				'left': '0px',
			}, 200 );

			$( '#happyforms-form-name', this.$el ).focus();
		},

		onNameChange: function( e ) {
			e.preventDefault();

			var value = $( e.target ).val();
			this.model.set( 'post_title', value );
			api.previewer.send( 'happyforms-form-title-update', value );
		},

		onExpandCollapseAllClick: function(e) {
			e.preventDefault();

			var $button = $(e.target);

			this.partViews.each(function (model) {
				if ($button.hasClass('expand')) {
					model.get('view').trigger('widget-expand');
				} else {
					model.get('view').trigger('widget-collapse');
				}
			});

			if ($button.hasClass('expand')) {
				$button.text($button.data('collapse-text')).removeClass('expand').addClass('collapse');
			} else {
				$button.text($button.data('expand-text')).removeClass('collapse').addClass('expand');
			}
		},

		showExpandCollapseButton: function() {
			this.$el.find('.expand-collapse-all').show();
		},

		hideExpandCollapseButton: function () {
			this.$el.find('.expand-collapse-all').hide();
		},

		onPartAdd: function( type, options ) {
			var self = this;
			var partModel = PartFactory.model(
				{ type: type },
				{ collection: this.model.get( 'parts' ) },
			);

			this.model.get( 'parts' ).add( partModel, options );
			this.model.trigger( 'change', this.model );

			partModel.fetchHtml( function( response ) {
				if ( happyForms.previewer.ready ) {
					api.previewer.send( 'happyforms-form-part-add', response );
				} else {
					happyForms.buffer.push( {
						event: 'happyforms-form-part-add',
						data: response
					} );
				}
			} );
		},

		onPartDuplicate: function( part, options ) {
			var attrs = part.toJSON();
			delete attrs.id;
			attrs.label += ' (Copy)';

			var duplicate = PartFactory.model(
				attrs,
				{ collection: this.model.get( 'parts' ) },
			);

			this.model.get( 'parts' ).add( duplicate, options );
			this.model.trigger( 'change', this.model );

			duplicate.fetchHtml( function( response ) {
				if ( happyForms.previewer.ready ) {
					api.previewer.send( 'happyforms-form-part-add', response );
				} else {
					happyForms.buffer.push( {
						event: 'happyforms-form-part-add',
						data: response
					} );
				}
			} );
		},

		onPartModelAdd: function( partModel, partsCollection, options ) {
			this.addViewPart( partModel, options );
		},

		onPartModelRemove: function( partModel ) {
			this.model.trigger( 'change', this.model );

			var partViewModel = this.partViews.find( function( viewModel ) {
				return viewModel.get( 'view' ).model.id === partModel.id;
			}, this );

			this.partViews.remove( partViewModel );

			if ( happyForms.previewer.ready ) {
				api.previewer.send( 'happyforms-form-part-remove', partModel.id );
			} else {
				happyForms.buffer.push( {
					event: 'happyforms-form-part-remove',
					data: partModel.id
				} );
			}
		},

		onPartModelChange: function( partModel ) {
			this.model.trigger( 'change' );
		},

		onPartModelsSorted: function() {
			this.partViews.reset( _.map( this.model.get( 'parts' ).pluck( 'id' ), function( id ) {
				return this.partViews.get( id );
			}, this ) );
			this.model.trigger( 'change' );

			var ids = this.model.get( 'parts' ).pluck( 'id' );
			api.previewer.send( 'happyforms-form-parts-sort', ids );
		},

		addViewPart: function( partModel, options ) {
			var settings = happyForms.parts.findWhere( { type: partModel.get( 'type' ) } );

			var partView = PartFactory.view( _.extend( {
				type: settings.get( 'type' ),
				model: partModel,
				settings: settings,
			}, options ) );

			var partViewModel = new Backbone.Model( {
				id: partModel.id,
				view: partView,
			} );

			this.partViews.add( partViewModel, options );
		},

		onPartViewAdd: function( viewModel, collection, options ) {
			var partView = viewModel.get( 'view' );
			$( '.happyforms-form-widgets', this.$el ).append( partView.render().$el );
			partView.trigger( 'ready' );

			if ( options.scrollto ) {
				this.$el.parent().animate( {
					scrollTop: partView.$el.position().top
				}, 400 );
			}

			this.showExpandCollapseButton();
		},

		onPartViewRemove: function( viewModel ) {
			var partView = viewModel.get( 'view' );
			partView.remove();

			if (!this.partViews.length) {
				this.hideExpandCollapseButton();
			}
		},

		onPartSortStop: function( e, ui ) {
			var $sortable = $( '.happyforms-form-widgets', this.$el );
			var ids = $sortable.sortable( 'toArray', { attribute: 'data-part-id' } );

			this.model.get( 'parts' ).reset( _.map( ids, function( id ) {
				return this.model.get( 'parts' ).get( id );
			}, this ) );
		},

		onPartViewsSorted: function( partViews ) {
			var $stage = $( '.happyforms-form-widgets', this.$el );

			partViews.forEach( function( partViewModel ) {
				var partView = partViewModel.get( 'view' );
				var $partViewEl = partView.$el;
				$partViewEl.detach();
				$stage.append( $partViewEl );
				partView.trigger( 'refresh' );
			}, this );
		},

		onPartViewsChanged: function( partViews ) {
			if ( this.partViews.length > 0 ) {
				this.$el.addClass( 'has-parts' );
			} else {
				this.$el.removeClass( 'has-parts' );
			}
		},

		remove: function() {
			while ( partView = this.partViews.first() ) {
				this.partViews.remove( partView );
			};

			var drawer = this.drawer;
			this.drawer.$el.animate( {
				'left': '-300px',
			}, 200, function() {
				drawer.remove();
			} );

			$( 'body' ).removeClass( 'adding-happyforms-parts' );

			classes.views.Base.prototype.remove.apply( this, arguments );
		}
	} );

	classes.views.PartsDrawer = classes.views.Base.extend( {
		template: '#happyforms-form-parts-drawer-template',

		events: {
			'click .happyforms-parts-list-item': 'onListItemClick',
			'keyup #part-search': 'onPartSearch',
		},

		render: function() {
			this.setElement( this.template( { parts: happyForms.parts.toJSON() } ) );
			return this;
		},

		onListItemClick: function( e ) {
			var type = $( e.currentTarget ).data( 'part-type' );
			happyForms.trigger( 'part-add', type, { expand: true } );
		},

		onPartSearch: function( e ) {
			var search = $( e.target ).val().toLowerCase();
			var $partEls = $( '.happyforms-parts-list-item', this.$el );

			if ( '' === search ) {
				$partEls.show();
			}

			var results = happyForms.parts.filter( function( part ) {
				var label = part.get( 'label' ).toLowerCase();
				var description = part.get( 'description' ).toLowerCase();

				return label.indexOf( search ) >= 0 || description.indexOf( search ) >= 0;
			} );

			$partEls.hide();

			results.forEach( function( part ) {
				$( '.happyforms-parts-list-item[data-part-type="' + part.get( 'type' ) + '"]', this.$el ).show();
			} );

		}
	} );

	classes.views.Part = classes.views.Base.extend( {
		$: $,

		events: {
			'click .happyforms-widget-action': 'onWidgetToggle',
			'click .happyforms-form-part-close': 'onWidgetToggle',
			'click .happyforms-form-part-remove': 'onPartRemoveClick',
			'click .happyforms-form-part-duplicate': 'onPartDuplicateClick',
			'keyup [data-bind]': 'onInputChange',
			'change [data-bind]': 'onInputChange',
			'mouseover': 'onMouseOver',
			'mouseout': 'onMouseOut',
			'change .apply-all-check': 'applyOptionGlobally',
			'click .happyforms-form-part-advanced-settings': 'onAdvancedSettingsClick'
		},

		initialize: function( options ) {
			classes.views.Base.prototype.initialize.apply( this, arguments );
			this.settings = options.settings;

			// listen to changes in common settings
			this.listenTo( this.model, 'change:label', this.onPartLabelChange );
			this.listenTo( this.model, 'change:width', this.onPartWidthChange );
			this.listenTo( this.model, 'change:required', this.onRequiredCheckboxChange );
			this.listenTo( this.model, 'change:placeholder', this.onPlaceholderChange );
			this.listenTo( this.model, 'change:description', this.onDescriptionChange );
			this.listenTo( this.model, 'change:tooltip_description', this.onTooltipDescriptionChange );
			this.listenTo( this.model, 'change:label_placement', this.onLabelPlacementChange );
			this.listenTo( this.model, 'change:css_class', this.onCSSClassChange );
			this.listenTo( this, 'reveal-tooltip-checkbox', this.onRevealTooltipCheckbox );
			this.listenTo( this, 'conceal-tooltip-checkbox', this.onConcealTooltipCheckbox );

			this.listenTo( this, 'widget-expand', this.expand );
			this.listenTo( this, 'widget-collapse', this.collapse );

			if ( options.expand ) {
				this.listenTo( this, 'ready', this.expandToggle );
			}
		},

		render: function() {
			this.setElement( this.template( {
				settings: this.settings.toJSON(),
				instance: this.model.toJSON(),
			} ) );

			return this;
		},

		/**
		 * Trigger a previewer event on mouse over.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onMouseOver: function() {
			var data = {
				id: this.model.id,
				callback: 'onPartMouseOverCallback',
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		/**
		 * Trigger a previewer event on mouse out.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onMouseOut: function() {
			var data = {
				id: this.model.id,
				callback: 'onPartMouseOutCallback',
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		/**
		 * Send changed label value to previewer.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onPartLabelChange: function() {
			var data = {
				id: this.model.id,
				callback: 'onPartLabelChangeCallback',
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		/**
		 * Send data about changed part width to previewer.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onPartWidthChange: function( model, value, options ) {
			var data = {
				id: this.model.id,
				callback: 'onPartWidthChangeCallback',
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		/**
		 * Trigger a previewer event on change of the "This is a required field" checkbox.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onRequiredCheckboxChange: function() {
			var data = {
				id: this.model.id,
				callback: 'onRequiredCheckboxChangeCallback',
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		/**
		 * Slide toggle part view in the customize pane.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		expandToggle: function() {
			var $el = this.$el;
			var self = this;

			$( '.happyforms-widget-content', this.$el ).slideToggle( function() {
				$el.toggleClass('happyforms-widget-expanded');

				self.model.collection.trigger('widget-toggle');
			} );
		},

		/**
		 * Expand part view.
		 *
		 * @since 1.1.0.
		 *
		 * @return void
		 */
		expand: function() {
			var $el = this.$el;

			$el.find('.happyforms-widget-content').slideDown(function() {
				$el.addClass('happyforms-widget-expanded');
			});
		},

		/**
		 * Collapse part view.
		 *
		 * @since 1.1.0.
		 *
		 * @return void
		 */
		collapse: function() {
			var $el = this.$el;

			$el.find('.happyforms-widget-content').slideUp(function () {
				$el.removeClass('happyforms-widget-expanded');
			});
		},

		/**
		 * Call expandToggle method on toggle indicator click or 'Close' button click of the part view in Customize pane.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onWidgetToggle: function( e ) {
			e.preventDefault();
			this.expandToggle();
		},

		/**
		 * Remove part model from collection on "Delete" button click.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onPartRemoveClick: function( e ) {
			e.preventDefault();

			this.model.collection.remove( this.model );
		},

		onPartDuplicateClick: function( e ) {
			e.preventDefault();

			happyForms.trigger( 'part-duplicate', this.model, {
				expand: true,
				scrollto: true,
			} );
		},

		/**
		 * Update model with the changed data. Triggered on change event of inputs in the part view.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onInputChange: function( e ) {
			var $el = $( e.target );
			var value = $el.val();
			var attribute = $el.data( 'bind' );

			if ( 'label' === attribute ) {
				var $inWidgetTitle = this.$el.find('.in-widget-title');
				$inWidgetTitle.find('span').text(value);

				if ( value ) {
					$inWidgetTitle.show();
				} else {
					$inWidgetTitle.hide();
				}
			}

			if ( $el.is(':checkbox') ) {
				if ( $el.is(':checked') ) {
					value = 1;
				} else {
					value = 0;
				}
			}

			this.model.set( attribute, value );
		},

		/**
		 * Send changed placeholder value to previewer.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onPlaceholderChange: function() {
			var data = {
				id: this.model.id,
				callback: 'onPlaceholderChangeCallback',
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		/**
		 * Send changed description value to previewer.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onDescriptionChange: function( model, value ) {
			var data = {
				id: this.model.id,
				callback: 'onDescriptionChangeCallback',
			};

			if ( value ) {
				this.trigger('reveal-tooltip-checkbox');
			} else {
				model.set('tooltip_description', 0);
				this.trigger('conceal-tooltip-checkbox');
			}

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		/**
		 * Trigger a previewer event on tooltip description checkbox change.
		 *
		 * @since 1.1.0.
		 *
		 * @return void
		 */
		onTooltipDescriptionChange: function() {
			var data = {
				id: this.model.id,
				callback: 'onTooltipDescriptionChangeCallback',
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		/**
		 * Send data about changed label placement value to previewer.
		 *
		 * @since 1.0.0.
		 *
		 * @return void
		 */
		onLabelPlacementChange: function( model, value, options ) {
			var $select = $( '[data-bind=label_placement]', this.$el );

			if ( $('option[value='+value+']', $select).length > 0 ) {
				$select.val( value );

				if ( ! options.skipGlobalReveal ) {
					var $globalWrap = this.$( '.label_placement-options', this.$el );
					// reset global checkbox
					this.$( 'input', $globalWrap ).prop( 'checked', false );
					// fade in the global checkbox wrapper
					$globalWrap.fadeIn();
				}

				model.fetchHtml( function( response ) {
					var data = {
						id: model.get( 'id' ),
						html: response,
					};

					api.previewer.send( 'happyforms-form-part-refresh', data );
				} );
			} else {
				model.set('label_placement', model.previous('label_placement'), { silent: true });
			}
		},

		onCSSClassChange: function( model, value, options ) {
			var data = {
				id: this.model.id,
				callback: 'onCSSClassChangeCallback',
				options: options,
			};

			api.previewer.send( 'happyforms-part-dom-update', data );
		},

		applyOptionGlobally: function(e) {
			var $input = $(e.target);
			var model = this.model;
			var modelAttribute = $input.attr('data-apply-to');
			var newValue = this.model.get(modelAttribute);

			_(happyForms.form.get('parts').models).each(function (partModel) {
				if (partModel.id !== model.id) {
					if ($input.is(':checked')) {
						partModel.set(modelAttribute, newValue, { skipGlobalReveal: true });
					} else {
						partModel.set(modelAttribute, partModel.previousAttributes()[modelAttribute], { skipGlobalReveal: true });
					}
				}
			});
		},

		onRevealTooltipCheckbox: function() {
			this.$el.find('.tooltip-description-wrap').fadeIn();
		},

		onConcealTooltipCheckbox: function() {
			var $descriptionWrap = this.$el.find('.tooltip-description-wrap');

			$descriptionWrap.fadeOut(200, function() {
				$descriptionWrap.find('input').prop('checked', false);
			});
		},

		onAdvancedSettingsClick: function(e) {
			var $link = $(e.target);
			var $advancedSettingsWrap = this.$el.find('.happyforms-part-advanced-settings-wrap');

			if ( $advancedSettingsWrap.is(':hidden') ) {
				$advancedSettingsWrap.slideDown(300, function() {
					$link.addClass('opened');
				});
			} else {
				$advancedSettingsWrap.slideUp(300, function() {
					$link.removeClass('opened');
				});
			}
		}
	} );

	classes.views.FormSetup = classes.views.Base.extend( {
		template: '#happyforms-form-setup-template',

		events: {
			'keyup [data-attribute]': 'onInputChange',
			'change [data-attribute]': 'onInputChange',
			'mouseover [data-pointer]': 'onHelpMouseOver',
			'mouseout [data-pointer]': 'onHelpMouseOut',
		},

		pointers: {},
		editorIds: [
			'happyforms-confirmation-message',
			'happyforms-confirmation-email-content'
		],

		initialize: function() {
			classes.views.Base.prototype.initialize.apply( this, arguments );

			this.listenTo( this.model, 'change:receive_email_alerts', this.onReceiveEmailAlertsChange );
			this.listenTo( this.model, 'change:send_confirmation_email', this.onSendEmailConfirmationChange );
			this.listenTo( this.model, 'change:captcha', this.onChangeCaptcha );
			this.listenTo( this.model, 'change:captcha_site_key', this.onChangeCaptchaKey );
			this.listenTo( this.model, 'change:captcha_secret_key', this.onChangeCaptchaKey );
			this.listenTo( this.model, 'change:submit_button_label', this.onSubmitButtonLabelChange );
		},

		render: function() {
			this.setElement( this.template( this.model.toJSON() ) );
			return this;
		},

		ready: function() {
			this.setupHelpPointers();

			var editorSettings = {
				tinymce: {
					toolbar1: 'bold,italic,bullist,numlist,link,hr',
					setup: this.onEditorInit.bind( this ),
					content_style: 'body { font-family: sans-serif; }'
				},
			};

			_.each( this.editorIds, function( editorId ) {
				var $textarea = $( '#' + editorId, this.$el );
				wp.editor.initialize( editorId, editorSettings );
			} );
		},

		setupHelpPointers: function() {
			var $helpTriggers = $( '[data-pointer]', this.$el );
			var self = this;

			$helpTriggers.each( function() {
				var $trigger = $( this );
				var pointerId = $trigger.data( 'pointer' );
				var $target = $trigger.parents( '.customize-control' ).find( '[data-pointer-target]' );
				var $pointerTemplate = $( '#happyforms-pointer-' + pointerId );

				var $pointer = $target.pointer( {
					pointerClass: 'wp-pointer happyforms-help-pointer',
					content: $pointerTemplate.html(),
					position: {
						edge: 'left',
						align: 'center',
					},
					open: function( e, ui ) {
						ui.pointer.css( 'margin-left', '-1px' );
					},
					close: function( e, ui ) {
						ui.pointer.css( 'margin-left', '0' );
					},
					buttons: function() {},
				} );

				self.pointers[pointerId] = $pointer;
			} );
		},

		onEditorInit: function( editor ) {
			var $textarea = $( '#' + editor.id, this.$el );
			var attribute = $textarea.data( 'attribute' );
			var self = this;

			editor.on( 'keyup change', function() {
				self.model.set( attribute, editor.getContent() );
			} );
		},

		onInputChange: function( e ) {
			e.preventDefault();

			var $el = $( e.target );
			var attribute = $el.data( 'attribute' );
			var value = $el.val();

			if ( $el.is( ':checkbox' ) ) {
				value = $el.is( ':checked' ) ? value: 0;
			}

			this.model.set( attribute, value );
		},

		onHelpMouseOver: function( e ) {
			var $target = $( e.target );
			var pointerId = $target.data( 'pointer' );
			var $pointer = this.pointers[pointerId];

			if ( $pointer ) {
				$pointer.pointer( 'open' );
			}
		},

		onHelpMouseOut: function( e ) {
			var $target = $( e.target );
			var pointerId = $target.data( 'pointer' );
			var $pointer = this.pointers[pointerId];

			if ( $pointer ) {
				$pointer.pointer( 'close' );
			}
		},

		onReceiveEmailAlertsChange: function( model, value ) {
			var $alertEmailSettings = $( '#happyforms-alert-email-settings', this.$el );

			if ( value ) {
				$alertEmailSettings.show();
			} else {
				$alertEmailSettings.hide();
			}
		},

		onSendEmailConfirmationChange: function( model, value ) {
			var $confirmationEmailSettings = $( '#happyforms-confirmation-email-settings', this.$el );

			if ( value ) {
				$confirmationEmailSettings.show();
			} else {
				$confirmationEmailSettings.hide();
			}
		},

		onChangeCaptcha: function() {
			var $captchaSettings = $( '#happyforms-captcha-settings', this.$el );

			if ( this.model.get( 'captcha' ) ) {
				$captchaSettings.show();
			} else {
				$captchaSettings.hide();
			}

			this.onChangeCaptchaKey();
		},

		onChangeCaptchaKey: function() {
			var data = {
				callback: 'onRecaptchaUpdate',
			};

			if ( happyForms.previewer.ready ) {
				api.previewer.send( 'happyforms-form-recaptcha-update', data );
			} else {
				happyForms.buffer.push( {
					event: 'happyforms-form-recaptcha-update',
					data: data
				} );
			}
		},

		onSubmitButtonLabelChange: function( model, value ) {
			api.previewer.send( 'happyforms-submit-button-text-update', value );
		},

		remove: function() {
			_.each( this.editorIds, function( editorId ) {
				wp.editor.remove( editorId );
			} );

			classes.views.Base.prototype.remove.apply( this, arguments );
		}
	} );

	classes.views.FormStyle = classes.views.Base.extend( {
		template: '#happyforms-form-style-template',

		events: {
			'click h3.accordion-section-title': 'onGroupClick',
			'click .customize-panel-back': 'onGroupBackClick',
			'change [data-target="form_class"] input': 'onFormClassChange',
			'change [data-target="form_class"] select': 'onFormClassChange',
			'change [data-target="form_class"] input[type="checkbox"]': 'onFormClassCheckboxChange',
			'change [data-target="css_var"] input[type=radio]': 'onButtonSetCssVarChange'
		},

		initialize: function() {
			classes.views.Base.prototype.initialize.apply( this, arguments );

			this.styles = new Backbone.Collection();
		},

		render: function() {
			this.setElement( this.template( this.model.toJSON() ) );
			return this;
		},

		ready: function() {
			this.initColorPickers();
			this.initUISliders();
			this.initButtonSet();
			this.initFormWidthSlider();
		},

		onFormClassChange: function( e ) {
			e.preventDefault();

			var $target = $( e.target );
			var attribute = $target.data( 'attribute' );
			var value = $target.val();

			happyForms.form.set( attribute, value );

			var data = {
				attribute: attribute,
				callback: 'onFormClassChangeCallback',
			};

			api.previewer.send( 'happyforms-form-class-update', data );
		},

		onFormClassCheckboxChange: function( e ) {
			e.preventDefault();

			var $target = $( e.target );
			var attribute = $target.data( 'attribute' );
			var value = $target.val();

			if ( $target.is(':checked') ) {
				happyForms.form.set( attribute, value );
			} else {
				happyForms.form.set( attribute, '' );
			}

			var data = {
				attribute: attribute,
				callback: 'onFormClassToggleCallback'
			};

			api.previewer.send( 'happyforms-form-class-update', data );
		},

		onButtonSetCssVarChange: function( e ) {
			e.preventDefault();

			var $target = $(e.target);
			var attribute = $target.data('attribute');
			var variable = $target.parents('.happyforms-buttonset-control').data('variable');

			var value = $target.val();

			happyForms.form.set(attribute, value);

			var data = {
				variable: variable,
				value: value,
			};

			api.previewer.send('happyforms-css-variable-update', data);
		},

		onGroupClick: function( e ) {
			e.preventDefault();

			$( '.happyforms-style-controls-group', this.$el ).removeClass( 'open' );

			$( '.happyforms-divider-control', this.$el )
				.removeClass( 'active' )
				.addClass( 'inactive' );

			$( e.target ).parent().next().addClass( 'open' );
		},

		onGroupBackClick: function( e ) {
			e.preventDefault();

			$( '.happyforms-divider-control', this.$el )
				.removeClass( 'inactive' )
				.addClass( 'active' );

			var $section = $( e.target ).closest( '.happyforms-style-controls-group' );

			$section.addClass( 'closing' );

			setTimeout(function () {
				$section.removeClass('closing open');
			}, 200);
		},

		initButtonSet: function() {
			$('.happyforms-buttonset-container').buttonset();
		},

		initColorPickers: function() {
			var self = this;
			var $colorInputs = $( '.happyforms-color-input', this.$el );

			$colorInputs.each( function( index, el ) {
				var $control = $( el ).parents( '.customize-control' );
				var variable = $control.data( 'variable' );

				$( el ).wpColorPicker( {
					change: function( e, ui ) {
						var value = ui.color.toString();

						self.model.set( $( this ).attr( 'data-attribute' ), value );

						var data = {
							variable: variable,
							value: value,
						};

						api.previewer.send( 'happyforms-css-variable-update', data );
					}
				} );

				var $wpPickerContainer = $( el ).parent().parent();

				$wpPickerContainer.find( '.wp-picker-clear' ).on( 'click', function() {
					var attribute = $( el ).attr( 'data-attribute' );
					var value = $( el ).attr( 'data-default' );

					self.model.set( attribute, value );
				} );
			} );
		},

		initUISliders: function() {
			var self = this;
			var $container = this.$el.find( '.happyforms-range-control:not(.form-width-control)' );

			$container.each(function (el, index) {
				var $this = $(this);
				var variable = $this.data('variable');
				var $slider = $( '.happyforms-range-slider', $this );
				var $sliderInput = $( 'input', $this );
				var min = parseFloat( $sliderInput.attr( 'min' ) );
				var max = parseFloat( $sliderInput.attr( 'max' ) );
				var step = parseFloat( $sliderInput.attr( 'step' ) );
				var value = parseFloat( $sliderInput.val() );

				$sliderInput.on('keyup', function() {
					var $this = $(this);

					self.model.set( $sliderInput.attr('data-attribute'), $this.val() );

					var data = {
						variable: variable,
						value: $this.val() + '' + $this.parent().attr('data-unit'),
					};

					api.previewer.send('happyforms-css-variable-update', data);
				});

				$slider.slider( {
					value: value,
					min: min,
					max: max,
					step: step,

					stop: function( e, ui ) {
						$sliderInput.val(ui.value);
						self.model.set( $sliderInput.attr( 'data-attribute' ), ui.value );

						var data = {
							variable: variable,
							value: ui.value + '' + $sliderInput.parent().attr('data-unit'),
						};

						api.previewer.send('happyforms-css-variable-update', data);
					}
				} );
			} );
		},

		initFormWidthSlider: function(reInit) {
			var self = this;

			var $container = this.$el.find('.happyforms-range-control.form-width-control');
			var $slider = $('.happyforms-range-slider', $container);
			var $sliderInput = $('input', $container);
			var $unitSwitch = $('.happyforms-unit-switch', $container);

			var value = ($sliderInput.val()) ? parseFloat($sliderInput.val()) : 100;
			var currentWidth = this.model.get('form_width').toString();
			var variable = $container.data('variable');
			var unit = $unitSwitch.val();

			if ( typeof reInit === 'undefined' || !reInit ) {
				if ( currentWidth.indexOf('%') !== -1 ) {
					unit = '%';
				} else if ( currentWidth.indexOf('px') !== -1 ) {
					unit = 'px';
				} else {
					unit = '%';
				}

				$unitSwitch.val(unit);
			}

			var min = (unit === 'px') ? 360 : 0;
			var max = (unit === 'px') ? 1440 : 100;
			var step = (unit === 'px') ? 10 : 5;

			$sliderInput.attr('min', min);
			$sliderInput.attr('max', max);
			$sliderInput.attr('step', step);

			$unitSwitch.on('change', function () {
				self.initFormWidthSlider(true);
			});

			if (typeof reInit !== 'undefined' && reInit) {
				value = (unit === '%') ? 100 : 900;

				self.updateFormWidth(value, unit, $slider);
			}

			$sliderInput.val(value);

			$sliderInput.on('keyup, change', function () {
				var $this = $(this);

				self.updateFormWidth($this.val(), unit, $slider);
			});

			$slider.slider({
				value: value,
				min: min,
				max: max,
				step: step,

				stop: function (e, ui) {
					$sliderInput.val(ui.value);

					self.updateFormWidth(ui.value, unit, $slider);
				}
			});
		},

		updateFormWidth: function( value, unit, $slider ) {
			$slider.slider('value', value);

			this.model.set('form_width', value + unit);

			var data = {
				variable: '--happyforms-form-width',
				value: value + unit,
			};

			api.previewer.send('happyforms-css-variable-update', data);
		},
	} );

	classes.views.FormDone = classes.views.Base.extend( {
		template: '#happyforms-form-done-template',

		events: _.extend( {}, classes.views.Base.prototype.events, {
			'click .happyforms-step-index': 'onStepIndexClick'
		} ),

		render: function( options ) {
			this.setElement( this.template() );
			return this;
		},

		onStepIndexClick: function( e ) {
			e.preventDefault();

			happyForms.navigate( 'forms', -1 );
		},
	} );

	Previewer = {
		$: $,
		ready: false,

		getPartModel: function( id ) {
			return happyForms.form.get( 'parts' ).get( id );
		},

		getPartElement: function( html ) {
			return this.$( html );
		},

		bind: function() {
			this.ready = true;

			// Form title pencil
			api.previewer.bind(
				'happyforms-pencil-click-title',
				this.onPreviewPencilClickTitle.bind( this )
			);

			// Part pencils
			api.previewer.bind(
				'happyforms-pencil-click-part',
				this.onPreviewPencilClickPart.bind( this )
			);
		},

		/**
		 *
		 * Previewer callbacks for pencils
		 *
		 */
		onPreviewPencilClickPart: function( id ) {
			happyForms.navigate( 'build', { trigger: true } );

			var $partWidget = $( '[data-part-id="' + id + '"]' );

			if ( ! $partWidget.hasClass( 'happyforms-widget-expanded' ) ) {
				$partWidget.find( '.toggle-indicator' ).click();
			}

			$( 'input', $partWidget ).first().focus();
		},

		onPreviewPencilClickTitle: function( id ) {
			happyForms.navigate( 'build', { trigger: true } );

			$( 'input[name="post_title"]' ).focus();
		},

		/**
		 *
		 * Previewer callbacks for live part DOM updates
		 *
		 */
		onPartMouseOverCallback: function( id, html ) {
			var $part = this.$( html );
			$part.addClass( 'highlighted' );
		},

		onPartMouseOutCallback: function( id, html ) {
			var $part = this.$( html );
			$part.removeClass( 'highlighted' );
		},

		onPartLabelChangeCallback: function( id, html ) {
			var part = happyForms.form.get( 'parts' ).get( id );
			var $part = this.$( html );
			var $label = this.$( '.happyforms-part__label span.label', $part ).first();

			$label.text( part.get( 'label' ) );
		},

		onRequiredCheckboxChangeCallback: function( id, html ) {
			var part = happyForms.form.get( 'parts' ).get( id );
			var $part = this.$( html );
			var required = part.get( 'required' );
			var $required = this.$( '.happyforms-required', $part );

			if ( parseInt(required, 10 ) === 1) {
				$required.show();
			} else {
				$required.hide();
			}
		},

		onPartWidthChangeCallback: function( id, html, options ) {
			var part = happyForms.form.get( 'parts' ).get( id );
			var $part = this.$( html );
			var width = part.get( 'width' );

			$part.removeClass( 'happyforms-part--width-half' );
			$part.removeClass( 'happyforms-part--width-full' );
			$part.removeClass( 'happyforms-part--width-third' );
			$part.addClass( 'happyforms-part--width-' + width);
		},

		onPlaceholderChangeCallback: function( id, html ) {
			var part = happyForms.form.get( 'parts' ).get( id );
			var $part = this.$( html );

			this.$( 'input', $part ).attr( 'placeholder', part.get( 'placeholder' ) );
		},

		onDescriptionChangeCallback: function( id, html ) {
			var part = happyForms.form.get( 'parts' ).get( id );
			var $part = this.$( html );
			var description = part.get('description');
			var $description = this.$( '.happyforms-part__description', $part );

			$description.text(description);
		},

		onTooltipDescriptionChangeCallback: function( id, html ) {
			var part = happyForms.form.get( 'parts' ).get( id );
			var $part = this.$( html );
			var $description = this.$( '.happyforms-tooltip + .happyforms-part__description', $part );
			var $tooltip = this.$( '.happyforms-part__tooltip', $part );

			if ( part.get( 'tooltip_description' ) === 1 ) {
				$tooltip.show();
				$description.hide();
			} else {
				$tooltip.hide();
				$description.show();
			}
		},

		onLabelPlacementChangeCallback: function( id, html, options ) {
			var part = happyForms.form.get( 'parts' ).get( id );
			var $part = this.$( html );

			$part.removeClass( 'happyforms-part--label-above' );
			$part.removeClass( 'happyforms-part--label-below' );
			$part.removeClass( 'happyforms-part--label-left' );
			$part.removeClass( 'happyforms-part--label-right' );
			$part.removeClass( 'happyforms-part--label-inside' );
			$part.addClass( 'happyforms-part--label-' + part.get( 'label_placement' ) );
		},

		onCSSClassChangeCallback: function( id, html, options ) {
			var part = happyForms.form.get( 'parts' ).get( id );
			var $part = this.$( html );
			var previousClass = part.previous( 'css_class' );
			var currentClass = part.get( 'css_class' );

			$part.removeClass( previousClass );
			$part.addClass( currentClass );
		},

		onSubPartAdded: function( id, partHTML, optionHTML ) {
			var partView = happyForms.sidebar.current.partViews.get( id ).get( 'view' );
			partView.onSubPartAdded( id, partHTML, optionHTML );
		},

		onFormClassChangeCallback: function( attribute, html, options ) {
			var $form = this.$( html );
			var previousClass = happyForms.form.previous( attribute );
			var currentClass = happyForms.form.get( attribute );

			$form.removeClass( previousClass );
			$form.addClass( currentClass );
		},

		onFormClassToggleCallback: function( attribute, html, options ) {
			var $form = this.$( html );
			var previousClass = happyForms.form.previous( attribute );
			var currentClass = happyForms.form.get( attribute );

			$form.removeClass( previousClass );
			$form.addClass( currentClass );
		},

		onRecaptchaUpdate: function( $recaptcha, $ ) {
			var captcha = happyForms.form.get( 'captcha' );

			if ( captcha ) {
				var siteKey = happyForms.form.get( 'captcha_site_key' ) || 'null';
				$recaptcha.attr( 'data-sitekey', siteKey );
				$recaptcha.happyFormPart( 'render' );
			} else {
				$recaptcha.happyFormPart( 'reset' );
			}
		}
	};

	happyForms = window.happyForms = new HappyForms();
	happyForms.classes = classes;
	happyForms.factory = PartFactory;
	happyForms.previewer = Previewer;

	happyForms.utils = {
		uniqueId: function( prefix, collection ) {
			if ( collection ) {
				var increments = collection
					.pluck( 'id' )
					.map( function( id ) {
						var numberId = id.match( /_(\d+)$/ );
						numberId = numberId !== null ? parseInt( numberId[1] ): 0;
						return numberId;
					} )
					.sort( function( a, b ) {
						return b - a;
					} );

				var increment = increments.length ? increments[0] + 1 : 1;

				return prefix + increment;
			}

			return _.uniqueId( prefix );
		}
	};

	api.bind( 'ready', function() {
		happyForms.start();

		api.previewer.bind( 'ready', function() {
			happyForms.flushBuffer();
			happyForms.previewer.bind();
		} );
	} );

} ) ( jQuery, _, Backbone, wp.customize, _happyFormsSettings );
