<?php

class HappyForms_Email_Message {

	/**
	 * The sender address.
	 *
	 * @since 1.4.3
	 *
	 * @var string
	 */
	private $from;

	/**
	 * The reply-to address.
	 *
	 * @since 1.4.3
	 *
	 * @var string
	 */
	private $reply_to;

	/**
	 * The list of recipients.
	 *
	 * @since 1.4.3
	 *
	 * @var array
	 */
	private $recipients;

	/**
	 * The list of cc addresses.
	 *
	 * @since 1.4.3
	 *
	 * @var array
	 */
	private $ccs;

	/**
	 * The email subject.
	 *
	 * @since 1.4.3
	 *
	 * @var string
	 */
	private $subject;

	/**
	 * The email content type.
	 *
	 * @since 1.4.3
	 *
	 * @var string
	 */
	private $content_type = 'text/html';

	/**
	 * The email message content.
	 *
	 * @since 1.4.3
	 *
	 * @var string
	 */
	private $content;

	/**
	 * The submission message this email is linked to.
	 *
	 * @since 1.4.3
	 *
	 * @var array
	 */
	public $message;

	public function __construct( $message = array() ) {
		$this->from = '';
		$this->to = '';
		$this->ccs = array();
		$this->subject = '';
		$this->content = '';
		$this->message = $message;
	}

	public function set_senders( $senders = array() ) {
		if ( is_string( $senders ) ) {
			$senders = array( $senders );
		}

		/**
		 * Filter the list of senders for this email message.
		 *
		 * @since 1.4.3
		 *
		 * @param array $senders Current list of senders.
		 * @param array $message The submission this email was triggered from.
		 *
		 * @return array
		 */
		$senders = apply_filters( 'happyforms_email_senders', $senders, $this->message );

		if ( count( $senders ) ) {
			$senders = array_map( 'trim', $senders );

			$this->from = $senders[0];
			$this->reply_to = $senders[0];
			$this->ccs = array_slice( $senders, 1 );
		}
	}

	public function set_recipients( $recipients = array() ) {
		if ( is_string( $recipients ) ) {
			$recipients = array( $recipients );
		}

		/**
		 * Filter the list of recipients for this email message.
		 *
		 * @since 1.4.3
		 *
		 * @param array $recipients Current list of recipients.
		 * @param array $message    The submission this email was triggered from.
		 *
		 * @return array
		 */
		$recipients = apply_filters( 'happyforms_email_recipients', $recipients, $this->message );
		$this->recipients = array_map( 'trim', $recipients );
	}

	public function set_subject( $subject = '' ) {
		$subject = trim( $subject );

		/**
		 * Filter the subject for this email message.
		 *
		 * @since 1.4.3
		 *
		 * @param string $subject Current subject.
		 * @param array  $message    The submission this email was triggered from.
		 * @param array  $recipients The address this email is being sent to.
		 *
		 * @return string
		 */
		$subject = apply_filters( 'happyforms_email_subject', $subject, $this->message, $this->recipients );
		$this->subject = $subject;
	}

	public function set_content( $content = '' ) {
		$content = trim( $content );

		/**
		 * Filter the content for this email message.
		 *
		 * @since 1.4.3
		 *
		 * @param string $content    Current content.
		 * @param array  $message    The submission this email was triggered from.
		 * @param array  $recipients The address this email is being sent to.
		 *
		 * @return string
		 */
		$content = apply_filters( 'happyforms_email_content', $content, $this->message, $this->recipients );

		$this->content = $content;
	}

	private function get_headers() {
		$headers = array();

		array_push( $headers, 'From: ' . $this->from );
		array_push( $headers, 'Reply-To: ' . $this->reply_to );

		foreach ( $this->ccs as $cc ) {
			array_push( $headers, 'Cc: ' . $cc );
		}

		return $headers;
	}

	public function get_content_type() {
		/**
		 * Filter the content type for this email message.
		 *
		 * @since 1.4.3
		 *
		 * @param string $content_type Current content type.
		 * @param array  $message      The submission this email was triggered from.
		 *
		 * @return string
		 */
		$content_type = apply_filters( 'happyforms_email_content_type', $this->content_type, $this->message );

		return $content_type;
	}

	public function send() {
		add_filter( 'wp_mail_content_type', array( $this, 'get_content_type' ) );

		$headers = $this->get_headers();

		foreach ( $this->recipients as $recipient ) {
			$result = wp_mail( $recipient, $this->subject, $this->content, $headers );
		}

		remove_filter( 'wp_mail_content_type', array( $this, 'get_content_type' ) );

		return $result;
	}

}
