<?php

class HappyForms_Form_Controller {

	/**
	 * The singleton instance.
	 *
	 * @since 1.0
	 *
	 * @var HappyForms_Form_Controller
	 */
	private static $instance;

	/**
	 * The form post type slug.
	 *
	 * @since 1.0
	 *
	 * @var string
	 */
	public $post_type = 'happyform';

	/**
	 * The singleton constructor.
	 *
	 * @since 1.0
	 *
	 * @return HappyForms_Form_Controller
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		self::$instance->hook();

		return self::$instance;
	}

	/**
	 * Register hooks.
	 *
	 * @since 1.0
	 *
	 * @return void
	 */
	public function hook() {
		add_action( 'init', array( $this, 'register_post_type' ) );
		add_action( 'wp', array( $this, 'inject_new_form' ) );
		add_filter( 'single_template', array( $this, 'single_template' ), 20 );
		add_action( 'trashed_post', array( $this, 'trashed_post' ) );
		add_action( 'delete_post', array( $this, 'delete_post' ) );
		add_action( 'untrashed_post', array( $this, 'untrashed_post' ) );
		add_filter( 'happyforms_frontend_dependencies', array( $this, 'script_dependencies' ), 10, 2 );
		add_filter( 'happyforms_form_has_captcha', array( $this, 'has_captcha' ), 10, 2 );

		if ( is_customize_preview() ) {
			add_filter( 'happyforms_part_class', array( $this, 'part_class' ) );
			add_filter( 'happyforms_the_form_title', array( $this, 'form_title' ) );
		}
	}

	/**
	 * Action: register the form custom post type.
	 *
	 * @hooked action init
	 *
	 * @since 1.0
	 *
	 * @return void
	 */
	public function register_post_type() {
		$labels = array(
			'name'               => __( 'All Forms', 'happyforms' ),
			'singular_name'      => __( 'Form', 'happyforms' ),
			'add_new_item'       => __( 'Build form', 'happyforms' ),
			'edit_item'          => __( 'Edit form', 'happyforms' ),
			'new_item'           => __( 'Build form', 'happyforms' ),
			'view_item'          => __( 'View form', 'happyforms' ),
			'view_items'         => __( 'View forms', 'happyforms' ),
			'search_items'       => __( 'Search Forms', 'happyforms' ),
			'not_found'          => __( 'No form found', 'happyforms' ),
			'not_found_in_trash' => __( 'No forms found in Trash', 'happyforms' ),
			'all_items'          => __( 'All Forms', 'happyforms' ),
		);

		$args = array(
			'labels'          => $labels,
			'public'          => true,
			'show_in_menu'    => false,
			'query_var'       => true,
			'rewrite'         => array( 'slug' => 'happyform' ),
			'capability_type' => 'page',
			'has_archive'     => false,
			'hierarchical'    => false,
			'supports'        => array( 'author', 'custom-fields' ),
		);

		register_post_type( $this->post_type, $args );

		$tracking_status = happyforms_get_tracking()->get_status();

		if ( 1 === intval( $tracking_status['status'] ) ) {
			flush_rewrite_rules();
		}
	}

	/**
	 * Action: inject a virtual HappyForms post object
	 * if we're previewing a new form.
	 *
	 * @since 1.3
	 *
	 * @hooked action template_redirect
	 *
	 * @return void
	 */
	public function inject_new_form() {
		global $wp_query;

		if ( ! is_customize_preview() ) {
			return;
		}

		if ( ! isset( $wp_query->query['p'] ) ||
			! isset( $wp_query->query['post_type'] ) ) {
			return;
		}

		$queried_post_type = $wp_query->query['post_type'];
		$queried_post_id = intval( $wp_query->query['p'] );

		if ( $this->post_type !== $queried_post_type || 0 !== $queried_post_id ) {
			return;
		}

		// See https://barn2.co.uk/create-fake-wordpress-post-fly/
		$post = $this->create_virtual();
		$this->inject_virtual_post( $post );
	}

	/**
	 * Filter: filter the template path used for
	 * the Customize screen preview and frontend rendering.
	 *
	 * @since 1.0
	 *
	 * @hooked filter single_template
	 *
	 * @param $single_template The original template path.
	 *
	 * @return string
	 */
	public function single_template( $single_template ) {
		global $post;

		if ( $post->post_type == happyforms_get_form_controller()->post_type ) {
			if ( is_customize_preview() ) {
				$single_template = happyforms_get_include_folder() . '/templates/preview-form-edit.php';
			} else {
				$single_template = happyforms_get_include_folder() . '/templates/single-form.php';
			}
		}

		return $single_template;
	}

	public function get_post_fields() {
		$fields = array(
			'ID' => array(
				'default' => '0',
				'sanitize' => 'intval',
			),
			'post_title' => array(
				'default' => __( 'Untitled form', 'happyforms' ),
				'sanitize' => 'sanitize_text_field',
			),
			'post_status' => array(
				'default' => 'publish',
				'sanitize' => 'happyforms_sanitize_post_status',
			),
			'post_type' => array(
				'default' => $this->post_type,
				'sanitize' => 'sanitize_text_field',
			)
		);

		return $fields;
	}

	public function get_meta_fields() {
		global $current_user;

		$fields = array(
			'confirmation_message' => array(
				'default' => __( 'Your message has been successfully sent. We appreciate you contacting us and we’ll be in touch soon.', 'happyforms' ),
				'sanitize' => 'esc_html',
			),
			'receive_email_alerts' => array(
				'default' => 1,
				'sanitize' => 'happyforms_sanitize_checkbox'
			),
			'email_recipient' => array(
				'default' => ( $current_user->user_email ) ? $current_user->user_email : '',
				'sanitize' => 'happyforms_sanitize_emails',
			),
			'alert_email_subject' => array(
				'default' => __( 'You received a new message', 'happyforms' ),
				'sanitize' => 'sanitize_text_field',
			),
			'send_confirmation_email' => array(
				'default' => 1,
				'sanitize' => 'happyforms_sanitize_checkbox'
			),
			'confirmation_email_subject' => array(
				'default' => __( 'We received your message', 'happyforms' ),
				'sanitize' => 'sanitize_text_field',
			),
			'confirmation_email_content' => array(
				'default' => __( 'Your message has been successfully sent. We appreciate you contacting us and we’ll be in touch soon.', 'happyforms' ),
				'sanitize' => 'esc_html',
			),
			'redirect_url' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field',
			),
			'spam_prevention' => array(
				'default' => 1,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
			'submit_button_label' => array(
				'default' => __( 'Submit Form', 'happyforms' ),
				'sanitize' => 'sanitize_text_field',
			),
			'form_expiration_datetime' => array(
				'default' => date( 'Y-m-d H:i:s', time() + 3600 * 24 * 7 ),
				'sanitize' => 'happyforms_sanitize_datetime',
			),
			'save_entries' => array(
				'default' => 1,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
			'captcha' => array(
				'default' => '',
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
			'captcha_site_key' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field',
			),
			'captcha_secret_key' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field',
			),
		);

		/**
		 * Filter fields stored as form post meta.
		 *
		 * @since 1.3
		 *
		 * @param array $fields Registered post meta fields.
		 *
		 * @return array
		 */
		$fields = apply_filters( 'happyforms_meta_fields', $fields );

		return $fields;
	}

	/**
	 * Get the defaults and sanitization configuration
	 * for the fields of the form post object.
	 *
	 * @since 1.0
	 *
	 * @param string $group An optional subset of fields
	 *                      to retrieve configuration for.
	 *
	 * @return array
	 */
	public function get_fields( $group = '' ) {
		$fields = array();

		switch ( $group ) {
			case 'post':
				$fields = $this->get_post_fields();
				break;
			case 'meta':
				$fields = $this->get_meta_fields();
				break;
			default:
				$fields = array_merge(
					$this->get_post_fields(),
					$this->get_meta_fields()
				);
				break;
		}

		return $fields;
	}

	public function get_field( $field ) {
		$fields = $this->get_fields();

		if ( isset( $fields[$field] ) ) {
			return $fields[$field];
		}

		return null;
	}

	public function get_defaults( $group = '' ) {
		$fields = $this->get_fields( $group );
		$defaults = array_combine( array_keys( $fields ), wp_list_pluck( $fields, 'default' ) );

		return $defaults;
	}

	public function get_default( $field ) {
		$defaults = $this->get_defaults();

		if ( isset( $defaults[$field] ) ) {
			return $defaults[$field];
		}

		return null;
	}

	public function validate_field( &$value, $key ) {
		$field = $this->get_field( $key );

		if ( isset( $field['sanitize'] ) && is_callable( $field['sanitize'] ) ) {
			$callback = $field['sanitize'];
			$value = call_user_func( $callback, $value );
		};
	}

	/**
	 * Validate the form data submitted from the Customize screen.
	 *
	 * @since 1.0
	 *
	 * @param array $post_data The raw input form data.
	 *
	 * @return array
	 */
	public function validate_fields( $post_data = array() ) {
		$defaults = $this->get_defaults();
		$filtered = array_intersect_key( $post_data, $defaults );
		$validated = wp_parse_args( $post_data, $filtered );
		array_walk( $validated, array( $this, 'validate_field' ) );

		return $validated;
	}

	/**
	 * Creates a virtual form post object.
	 *
	 * @since 1.3
	 *
	 * @return WP_Post
	 */
	private function create_virtual() {
		$post_id = 0;
		$defaults = $this->get_defaults();

		$post = new stdClass();
		$post->ID = $post_id;
		$post->post_author = 1;
		$post->post_date = current_time( 'mysql' );
		$post->post_date_gmt = current_time( 'mysql', 1 );
		$post->post_title = $this->get_default( 'post_title' );
		$post->post_content = '';
		$post->post_status = 'publish';
		$post->comment_status = 'closed';
		$post->ping_status = 'closed';
		$post->post_name = '';
		$post->post_type = $this->post_type;
		$post->filter = 'raw';

		$wp_post = new WP_Post( $post );
		wp_cache_add( $post_id, $wp_post, 'posts' );

		return $wp_post;
	}

	/**
	 * Injects a virtual post object
	 * in the current query.
	 *
	 * @since 1.3
	 *
	 * @return WP_Post
	 */
	private function inject_virtual_post( $post ) {
		global $wp, $wp_query;

		$wp_query->post = $post;
		$wp_query->posts = array( $post );
		$wp_query->queried_object = $post;
		$wp_query->queried_object_id = 0;
		$wp_query->found_posts = 1;
		$wp_query->post_count = 1;
		$wp_query->max_num_pages = 1;
		$wp_query->is_page = false;
		$wp_query->is_singular = true;
		$wp_query->is_single = true;
		$wp_query->is_attachment = false;
		$wp_query->is_archive = false;
		$wp_query->is_category = false;
		$wp_query->is_tag = false;
		$wp_query->is_tax = false;
		$wp_query->is_author = false;
		$wp_query->is_date = false;
		$wp_query->is_year = false;
		$wp_query->is_month = false;
		$wp_query->is_day = false;
		$wp_query->is_time = false;
		$wp_query->is_search = false;
		$wp_query->is_feed = false;
		$wp_query->is_comment_feed = false;
		$wp_query->is_trackback = false;
		$wp_query->is_home = false;
		$wp_query->is_embed = false;
		$wp_query->is_404 = false;
		$wp_query->is_paged = false;
		$wp_query->is_admin = false;
		$wp_query->is_preview = false;
		$wp_query->is_robots = false;
		$wp_query->is_posts_page = false;
		$wp_query->is_post_type_archive = false;

		$GLOBALS['wp_query'] = $wp_query;
		$wp->register_globals();
	}

	/**
	 * Create a new form post object.
	 *
	 * @since 1.0
	 *
	 * @return int|string
	 */
	public function create() {
		$post_data = array_merge(
			$this->get_defaults( 'post' ),
			array( 'meta_input' => array(
				$this->get_defaults( 'meta' )
			) )
		);

		$result = wp_insert_post( wp_slash( $post_data ), true );

		if ( is_wp_error( $result ) ) {
			return $result;
		}

		$result = get_post( $result );

		return $result;
	}

	/**
	 * Get a list of forms.
	 *
	 * @since 1.0
	 *
	 * @param array   $post_ids A list of form IDs to fetch.
	 * @param boolean $only_id  Whether or not to limit the
	 *                          results to the ID field.
	 *
	 * @return array
	 */
	public function get( $post_ids = array(), $only_id = false ) {
		$query_params = array(
			'post_type'   => happyforms_get_form_controller()->post_type,
			'post_status' => 'publish',
			'posts_per_page' => -1,
		);

		$query_params['post__in'] = is_array( $post_ids ) ? $post_ids : array( $post_ids );

		if ( true === $only_id ) {
			$query_params['fields'] = 'ids';
		}

		if ( 0 !== $post_ids ) {
			$forms = get_posts( $query_params );
		} else {
			$forms = array( $this->create_virtual() );
		}

		if ( true === $only_id ) {
			return $forms;
		}

		$form_entries = array();

		foreach ( $forms as $form ) {
			$form_entries[] = $this->to_array( $form );
		}

		if ( ! is_array( $post_ids ) ) {
			if ( count( $form_entries ) > 0 ) {
				return $form_entries[0];
			} else {
				return false;
			}
		}

		return $form_entries;
	}

	/**
	 * Turn a form post object into an array.
	 *
	 * @since 1.0
	 *
	 * @param WP_Post $form The form post object.
	 *
	 * @return array
	 */
	public function to_array( $form ) {
		$form_array = $form->to_array();
		$meta_fields = $this->get_fields( 'meta' );
		$form_meta_keys = get_post_custom_keys( $form->ID );
		$form_meta_keys = is_null( $form_meta_keys ) ? array() : $form_meta_keys;

		foreach ( $meta_fields as $field_name => $field_settings ) {
			if ( in_array( $field_name, $form_meta_keys ) ) {
				// If this meta is defined, use its value.
				$form_array[$field_name] = get_post_meta( $form->ID, $field_name, true );
			} else {
				// Otherwise use the default value.
				$form_array[$field_name] = $field_settings['default'];
			}
		}

		$parts = array();
		$part_layout = get_post_meta( $form->ID, 'layout', true );

		if ( ! empty( $part_layout ) ) {
			foreach ( $part_layout as $part_id ) {
				$part = get_post_meta( $form->ID, $part_id, true );
				$parts[] = $part;
			}
		}

		$form_array['parts'] = $parts;

		return $form_array;
	}

	/**
	 * Update a form post object.
	 *
	 * @since 1.0
	 *
	 * @param array $form_data The raw input form data.
	 *
	 * @return array
	 */
	public function update( $form_data = array() ) {
		$validated_data = $this->validate_fields( $form_data );

		if ( isset( $validated_data['ID'] ) && 0 === $validated_data['ID'] ) {
			$form = $this->create();
			$validated_data['ID'] = $form->ID;
		}

		$post_data = array_intersect_key( $validated_data, $this->get_defaults( 'post' ) );
		$meta_data = array_intersect_key( $validated_data, $this->get_defaults( 'meta' ) );
		$update_data = array_merge( $post_data, array(
			'meta_input' => $meta_data
		) );

		$result = wp_update_post( $update_data, true );

		if ( is_wp_error( $result ) ) {
			return $result;
		}

		$part_layout = array();
		$parts_data = isset( $form_data['parts'] ) ? $form_data['parts'] : array();
		$library = happyforms_get_part_library();

		foreach ( $parts_data as $part_data ) {
			$validated_part = $library->validate_part( $part_data );

			if ( ! is_wp_error( $validated_part ) ) {
				$part_id = $part_data['id'];
				$part_layout[] = $part_id;
				update_post_meta( $result, $part_id, $validated_part );
			}
		}

		update_post_meta( $result, 'layout', $part_layout );
		$result = $this->to_array( get_post( $result ) );

		return $result;
	}

	/**
	 * Delete a form post object.
	 *
	 * @since 1.0
	 *
	 * @param int|string $form_id The ID of the form object.
	 *
	 * @return boolean|WP_Post
	 */
	public function delete( $form_id ) {
		$result = wp_delete_post( $form_id, true );

		return $result;
	}

	/**
	 * Action: remove form messages when a form is removed.
	 *
	 * @since 1.0
	 *
	 * @hooked action delete_post
	 *
	 * @param int|string $post_id The ID of the form object.
	 *
	 * @return void
	 */
	public function delete_post( $post_id ) {
		$post = get_post( $post_id );

		if ( $this->post_type !== $post->post_type ) {
			return;
		}

		$messages = get_posts( array(
			'post_type' => happyforms_get_message_controller()->post_type,
			'post_status' => 'any',
			'meta_key' => 'form_id',
			'meta_value' => $post_id,
		) );

		foreach ( $messages as $message ) {
			wp_delete_post( $message->ID, true );
		}

		happyforms_get_message_controller()->update_badge_transient();
	}

	/**
	 * Action: trigger an update to the unread messages badge
	 * when a form is trashed.
	 *
	 * @since 1.1
	 *
	 * @hooked action trashed_post
	 *
	 * @param int|string $post_id The ID of the form object.
	 *
	 * @return void
	 */
	public function trashed_post( $post_id ) {
		$post = get_post( $post_id );

		if ( $this->post_type !== $post->post_type ) {
			return;
		}

		happyforms_get_message_controller()->update_badge_transient();
	}

	/**
	 * Action: trigger an update to the unread messages badge
	 * when a form is restored.
	 *
	 * @since 1.1
	 *
	 * @hooked action untrashed_post
	 *
	 * @param int|string $post_id The ID of the form object.
	 *
	 * @return void
	 */
	public function untrashed_post( $post_id ) {
		$post = get_post( $post_id );

		if ( $this->post_type !== $post->post_type ) {
			return;
		}

		happyforms_get_message_controller()->update_badge_transient();
	}

	/**
	 * Return the first part with the given type found in a form.
	 *
	 * @since 1.0
	 *
	 * @param array  $form_data The data of the form the part belongs to.
	 * @param string $type      The type of the part.
	 *
	 * @return boolean|array
	 */
	public function get_first_part_by_type( $form_data, $type = '' ) {
		foreach( $form_data['parts'] as $part ) {
			if ( $type === $part['type'] ) {
				return $part;
				break;
			}
		}

		return false;
	}

	/**
	 * Get whether or not the given form data has spam prevention on.
	 *
	 * @since 1.0
	 *
	 * @param array $form_data The form data.
	 *
	 * @return int
	 */
	public function has_spam_protection( $form_data ) {
		return $form_data['spam_prevention'];
	}

	public function has_recaptcha_protection( $form_data ) {
		return $form_data['captcha']
			&& $form_data['captcha_site_key']
			&& $form_data['captcha_secret_key'];
	}

	/**
	 * Get form-wide submission notice definitions.
	 *
	 * @since 1.0
	 *
	 * @param array $form_data The form data.
	 *
	 * @return array
	 */
	public function get_message_definitions( $form_data ) {
		return array(
			'form_error' => array(
				'type' => 'error-submission',
				'message' => __( 'Your submission contains errors.', 'happyforms' ),
			),
			'form_success' => array(
				'type' => 'success',
				'message' => html_entity_decode( $form_data['confirmation_message'] ),
			),
		);
	}

	/**
	 * Get the HTML string of a rendered form.
	 *
	 * @since 1.0
	 *
	 * @param array $form The form data.
	 *
	 * @return string
	 */
	public function render( $form = array(), $render_styles = false ) {
		$form_markup = '';

		if ( empty( $form ) ) {
			return $form_markup;
		}

		ob_start();

		if ( $render_styles ) {
			happyforms_the_form_styles( $form );
		}

		require( happyforms_get_include_folder() . '/templates/single-form.php' );
		$form_markup = ob_get_clean();

		return $form_markup;
	}

	public function has_captcha( $has_captcha, $form ) {
		$has_captcha = $form['captcha'] || happyforms_is_preview();

		return $has_captcha;
	}

	/**
	 * Filter: append -editable class to part templates.
	 *
	 * @since  1.0
	 *
	 * @hooked filter happyforms_part_class
	 *
	 * @return void
	 */
	public function part_class( $classes ) {
		$classes[] = 'happyforms-block-editable happyforms-block-editable--part';

		return $classes;
	}

	public function form_title( $title ) {
		$before = '<div class="happyforms-block-editable happyforms-block-editable--title">';
		$after = '</div>';
		$title = "{$before}{$title}{$after}";

		return $title;
	}

	public function script_dependencies( $deps, $forms ) {
		$has_captcha = false;

		foreach ( $forms as $form ) {
			if ( $form['captcha'] ) {
				$has_captcha = true;
				break;
			}
		}

		if ( ! happyforms_is_preview() && ! $has_captcha ) {
			return $deps;
		}

		wp_register_script(
			'google-recaptcha',
			'https://www.google.com/recaptcha/api.js',
			array(), false, true
		);

		wp_register_script(
			'recaptcha',
			happyforms_get_plugin_url() . 'assets/js/frontend/recaptcha.js',
			array( 'google-recaptcha' ), false, true
		);

		$deps[] = 'recaptcha';

		return $deps;
	}

}

if ( ! function_exists( 'happyforms_get_form_controller' ) ):
/**
 * Get the HappyForms_Form_Controller class instance.
 *
 * @since 1.0
 *
 * @return HappyForms_Form_Controller
 */
function happyforms_get_form_controller() {
	return HappyForms_Form_Controller::instance();
}

endif;

/**
 * Initialize the HappyForms_Form_Controller class immediately.
 */
happyforms_get_form_controller();
