<?php

class HappyForms_Part_Date extends HappyForms_Form_Part {

	public $type = 'date';

	public function __construct() {
		$this->label = __( 'Date & Time', 'happyforms' );
		$this->description = __( 'For formatted day, month, year and or time fields.', 'happyforms' );

		add_filter( 'happyforms_part_class', array( $this, 'html_part_class' ), 10, 3 );
		add_filter( 'happyforms_part_value', array( $this, 'get_part_value' ), 10, 4 );
		add_filter( 'happyforms_stringify_part_value', array( $this, 'stringify_value' ), 10, 3 );
		add_filter( 'happyforms_frontend_dependencies', array( $this, 'script_dependencies' ), 10, 2 );
	}

	/**
	 * Get all part meta fields defaults.
	 *
	 * @since 1.0.0.
	 *
	 * @return array
	 */
	public function get_customize_fields() {
		return array(
			'type' => array(
				'default' => $this->type,
				'sanitize' => 'sanitize_text_field',
			),
			'label' => array(
				'default' => __( 'Date', 'happyforms '),
				'sanitize' => 'sanitize_text_field',
			),
			'label_placement' => array(
				'default' => 'above',
				'sanitize' => 'sanitize_text_field'
			),
			'description' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field'
			),
			'tooltip_description' => array(
				'default' => 0,
				'sanitize' => 'happyforms_sanitize_checkbox'
			),
			'date_type' => array(
				'default' => 'date',
				'sanitize' => 'sanitize_text_field'
			),
			'time_format' => array(
				'default' => 12,
				'sanitize' => 'intval'
			),
			'default_datetime' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field'
			),
			'min_year' => array(
				'default' => 1910,
				'sanitize' => 'intval'
			),
			'max_year' => array(
				'default' => intval( date( 'Y' ) ) + 20,
				'sanitize' => 'intval'
			),
			'years_option' => array(
				'default' => 'all',
				'sanitize' => 'sanitize_text_field'
			),
			'years_order' => array(
				'default' => 'asc',
				'sanitize' => 'sanitize_text_field'
			),
			'min_hour' => array(
				'default' => 1,
				'sanitize' => 'sanitize_text_field'
			),
			'max_hour' => array(
				'default' => 12,
				'sanitize' => 'intval'
			),
			'minute_step' => array(
				'default' => 1,
				'sanitize' => 'intval'
			),
			'width' => array(
				'default' => 'full',
				'sanitize' => 'sanitize_key'
			),
			'css_class' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field'
			),
			'required' => array(
				'default' => 1,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
		);
	}

	/**
	 * Get template for part item in customize pane.
	 *
	 * @since 1.0.0.
	 *
	 * @return string
	 */
	public function customize_templates() {
		require_once( happyforms_get_include_folder() . '/templates/parts/customize-date.php' );
	}

	/**
	 * Get front end part template with parsed data.
	 *
	 * @since 1.0.0.
	 *
	 * @param array	$part_data 	Form part data.
	 * @param array	$form_data	Form (post) data.
	 *
	 * @return string	Markup for the form part.
	 */
	public function frontend_template( $part_data = array(), $form_data = array() ) {
		$part = wp_parse_args( $part_data, $this->get_customize_defaults() );
		$form = $form_data;

		include( happyforms_get_include_folder() . '/templates/parts/frontend-date.php' );
	}

	public function get_default_value() {
		return array(
			'year' => '',
			'month' => '',
			'day' => '',
			'hour' => '',
			'minute' => '',
			'period' => '',
		);
	}

	/**
	 * Sanitize submitted value before storing it.
	 *
	 * @since 1.0.0.
	 *
	 * @param array $part_data Form part data.
	 *
	 * @return string
	 */
	public function sanitize_value( $part_data = array(), $form_data = array() ) {
		$sanitized_value = $this->get_default_value();
		$part_name = happyforms_get_part_name( $part_data, $form_data );

		if ( isset( $_REQUEST[$part_name] ) ) {
			$sanitized_value = wp_parse_args( $_REQUEST[$part_name], $sanitized_value );

			if ( '' !== implode( '', array_values( $sanitized_value ) ) ) {
				foreach( $sanitized_value as $component => $value ) {
					switch( $component ) {
						case 'year':
							$sanitized_value[$component] = sprintf( '%04d', intval( $value ) );
							break;
						case 'month':
						case 'day':
						case 'hour':
						case 'minute':
							$sanitized_value[$component] = sprintf( '%02d', intval( $value ) );
							break;
						case 'period':
							$sanitized_value[$component] = sanitize_text_field( $value );
							break;
					}
				}
			}
		}

		return $sanitized_value;
	}

	/**
	 * Validate value before submitting it. If it fails validation, return WP_Error object, showing respective error message.
	 *
	 * @since 1.0.0.
	 *
	 * @param array $part Form part data.
	 * @param string $value Submitted value.
	 *
	 * @return string|object
	 */
	public function validate_value( $value, $part = array(), $form = array() ) {
		$validated_value = $value;

		if ( 1 === $part['required'] && '' === implode( '', array_values( $validated_value ) ) ) {
			return new WP_Error( 'error', __( 'This field is required.', 'happyforms' ) );
		}

		$string_value = '';

		if ( $part['date_type'] === 'date' ) {
			$validated_value = array_slice( $validated_value, 0, 3, true );
			$string_value = vsprintf( '%s-%s-%s', array_values( $validated_value ) );
			$date_format = 'Y-m-d';
		} elseif ( $part['date_type'] === 'datetime' ) {
			if ( 12 == $part['time_format'] ) {
				$validated_value = array_slice( $validated_value, 0, 6, true );
				$string_value = vsprintf( '%s-%s-%s %s:%s %s', array_values( $validated_value ) );
				$date_format = 'Y-m-d h:i A';
			} else {
				$validated_value = array_slice( $validated_value, 0, 5, true );
				$string_value = vsprintf( '%04d-%s-%s %s:%s', array_values( $validated_value ) );
				$date_format = 'Y-m-d H:i';
			}
		} else {
			if ( 12 == $part['time_format'] ) {
				$validated_value = array_slice( $validated_value, 3, 3, true );
				$string_value = vsprintf( '%s:%s %s', array_values( $validated_value ) );
				$date_format = 'h:i A';
			} else {
				$validated_value = array_slice( $validated_value, 3, 2, true );
				$string_value = vsprintf( '%s:%s', array_values( $validated_value ) );
				$date_format = 'H:i';
			}
		}

		if ( ! date_create_from_format( $date_format, $string_value ) ) {
			return new WP_Error( 'error', __( 'Not a valid date.', 'happyforms' ) );
		} else {
			$date = strtotime( $string_value );

			if ( 'date' !== $part['date_type'] && isset( $part['min_hour'] ) && isset( $part['max_hour'] ) ) {
				$hour = (12 === $part['time_format']) ? intval( date( 'g', $date ) ) : intval( date( 'G', $date ) );
				$min_hour = intval( $part['min_hour'] );
				$max_hour = intval( $part['max_hour'] );

				if ( $hour > $max_hour || $hour < $min_hour ) {
					return new WP_Error(
						'error',
						__( 'Hour input does not match minimum and maximum value allowed.', 'happyforms' )
					);
				}
			}

			if ( 'time' !== $part['date_type'] && isset( $part['min_year'] ) && isset( $part['max_year'] ) ) {
				$year = intval( date( 'Y', $date ) );
				$min_year = intval( $part['min_year'] );
				$max_year = intval( $part['max_year'] );

				if ( $year > $max_year || $year < $min_year ) {
					return new WP_Error(
						'error',
						__( 'Year input does not match maximum value allowed.', 'happyforms' )
					);
				}
			}
		}

		return $validated_value;
	}

	public function get_part_value( $value, $part, $form, $component ) {
		if ( $this->type === $part['type'] ) {
			if ( false !== $component ) {
				$value = isset( $value[$component] ) ? $value[$component] : '';
			}
		}

		return $value;
	}

	public function stringify_value( $value, $part, $form ) {
		if ( $this->type === $part['type'] ) {
			extract( $value );

			$months = happyforms_get_months();
			$month = $months[intval( $month )];
			$day = intval( $day );

			if ( ! isset( $period ) ) {
				$period = '';
			}

			switch ( $part['date_type'] ) {
				case 'date':
					$value = "{$month} {$day}, {$year}";
					break;
				case 'time':
					$value = "{$hour}:{$minute}";
				case 'datetime':
					$value = "{$month} {$day}, {$year} at {$hour}:{$minute} {$period}";
					break;
				default:
					$value = '';
					break;
			}
		}

		return $value;
	}

	public function html_part_class( $class, $part, $form ) {
		if ( $this->type === $part['type'] ) {
			if ( isset( $part['date_type'] ) ) {
				$class[] = 'happyforms-part-date--' . $part['date_type'];
			}

			if ( isset( $part['time_format'] ) ) {
				$class[] = 'happyforms-part-date--' . $part['time_format'];
			}
		}

		return $class;
	}

	/**
	 * Enqueue scripts in customizer area.
	 *
	 * @since 1.0.0.
	 *
	 * @param array	List of dependencies.
	 *
	 * @return void
	 */
	public function customize_enqueue_scripts( $deps = array() ) {
		wp_enqueue_script(
			'part-date',
			happyforms_get_plugin_url() . 'assets/js/parts/part-date.js',
			$deps, HAPPYFORMS_VERSION, true
		);
	}

	public function script_dependencies( $deps, $forms ) {
		$contains_date = false;
		$form_controller = happyforms_get_form_controller();

		foreach ( $forms as $form ) {
			if ( $form_controller->get_first_part_by_type( $form, $this->type ) ) {
				$contains_date = true;
				break;
			}
		}

		if ( ! happyforms_is_preview() && ! $contains_date ) {
			return $deps;
		}

		wp_register_script(
			'happyforms-part-date',
			happyforms_get_plugin_url() . 'assets/js/frontend/date.js',
			array(), HAPPYFORMS_VERSION, true
		);

		$deps[] = 'happyforms-part-date';

		return $deps;
	}

}
