<?php

class HappyForms_Part_Table extends HappyForms_Form_Part {

	public $type = 'table';

	public function __construct() {
		$this->label = __( 'Table', 'happyforms' );
		$this->description = __( 'For radios and checkboxes displaying in a grid of rows and columns.', 'happyforms' );

		add_filter( 'happyforms_part_value', array( $this, 'get_part_value' ), 10, 4 );
		add_filter( 'happyforms_part_class', array( $this, 'html_part_class' ), 10, 3 );
		add_filter( 'happyforms_stringify_part_value', array( $this, 'stringify_value' ), 10, 3 );
		add_filter( 'happyforms_message_part_value', array( $this, 'message_part_value' ), 10, 4 );
		add_filter( 'happyforms_frontend_dependencies', array( $this, 'script_dependencies' ), 10, 2 );
	}

	/**
	 * Get all part meta fields defaults.
	 *
	 * @since 1.0.0.
	 *
	 * @return array
	 */
	public function get_customize_fields() {
		return array(
			'type' => array(
				'default' => $this->type,
				'sanitize' => 'sanitize_text_field',
			),
			'label' => array(
				'default' => __( 'Options', 'happyforms' ),
				'sanitize' => 'sanitize_text_field',
			),
			'label_placement' => array(
				'default' => 'above',
				'sanitize' => 'sanitize_text_field'
			),
			'description' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field'
			),
			'tooltip_description' => array(
				'default' => 0,
				'sanitize' => 'happyforms_sanitize_checkbox'
			),
			'width' => array(
				'default' => 'full',
				'sanitize' => 'sanitize_key'
			),
			'css_class' => array(
				'default' => '',
				'sanitize' => 'sanitize_text_field'
			),
			'required' => array(
				'default' => 1,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
			'allow_multiple_selection' => array(
				'default' => 0,
				'sanitize' => 'happyforms_sanitize_checkbox'
			),
			'columns' => array(
				'default' => array(),
				'sanitize' => 'happyforms_sanitize_array'
			),
			'rows' => array(
				'default' => array(),
				'sanitize' => 'happyforms_sanitize_array'
			),
		);
	}

	private function get_column_defaults() {
		return array(
			'is_default' => 0,
			'label' => '',
		);
	}

	private function get_row_defaults() {
		return array(
			'label' => '',
		);
	}

	/**
	 * Get template for part item in customize pane.
	 *
	 * @since 1.0.0.
	 *
	 * @return string
	 */
	public function customize_templates() {
		require_once( happyforms_get_include_folder() . '/templates/parts/customize-table.php' );
	}

	/**
	 * Get front end part template with parsed data.
	 *
	 * @since 1.0.0.
	 *
	 * @param array	$part_data 	Form part data.
	 * @param array	$form_data	Form (post) data.
	 *
	 * @return string	Markup for the form part.
	 */
	public function frontend_template( $part_data = array(), $form_data = array() ) {
		$part = wp_parse_args( $part_data, $this->get_customize_defaults() );
		$form = $form_data;

		foreach( $part['columns'] as $c => $column ) {
			$part['columns'][$c] = wp_parse_args( $column, $this->get_column_defaults() );
		}

		foreach( $part['rows'] as $r => $row ) {
			$part['rows'][$r] = wp_parse_args( $row, $this->get_row_defaults() );
		}

		include( happyforms_get_include_folder() . '/templates/parts/frontend-table.php' );
	}

	/**
	 * Enqueue scripts in customizer area.
	 *
	 * @since 1.0.0.
	 *
	 * @param array	List of dependencies.
	 *
	 * @return void
	 */
	public function customize_enqueue_scripts( $deps = array() ) {
		wp_register_script(
			'part-table',
			happyforms_get_plugin_url() . 'assets/js/parts/part-table.js',
			$deps, HAPPYFORMS_VERSION, true
		);

		$settings = array(
			'column' => $this->get_column_defaults(),
			'row' => $this->get_row_defaults(),
		);

		wp_localize_script( 'part-table', '_happyFormsTableSettings', $settings );
		wp_enqueue_script( 'part-table' );
	}

	public function get_default_value() {
		return array();
	}

	/**
	 * Sanitize submitted value before storing it.
	 *
	 * @since 1.0.0.
	 *
	 * @param array $part_data Form part data.
	 *
	 * @return string
	 */
	public function sanitize_value( $part_data = array(), $form_data = array() ) {
		$sanitized_value = $this->get_default_value();
		$part_name = happyforms_get_part_name( $part_data, $form_data );

		if ( isset( $_REQUEST[$part_name] ) ) {
			$requested_data = $_REQUEST[$part_name];

			if ( is_array( $requested_data ) ) {
				if ( $part_data['allow_multiple_selection'] ) {
					// Checkbox mode
					foreach( $requested_data as $row_id => $row ) {
						$sanitized_value[$row_id] = array_map( 'sanitize_text_field', $requested_data[$row_id] );
					}
				} else {
					// Radio mode
					$sanitized_value = array_map( 'sanitize_text_field', $requested_data );
				}
			}
		}

		return $sanitized_value;
	}

	/**
	 * Validate value before submitting it. If it fails validation, return WP_Error object, showing respective error message.
	 *
	 * @since 1.0.0.
	 *
	 * @param array $part Form part data.
	 * @param string $value Submitted value.
	 *
	 * @return string|object
	 */
	public function validate_value( $value, $part = array(), $form = array() ) {
		$validated_value = $value;

		if ( 1 === $part['required'] && empty( $validated_value ) ) {
			return new WP_Error( 'error', __( 'This field is required.', 'happyforms' ) );
		}

		$columns = happyforms_get_part_options( $part['columns'], $part, $form );
		$valid_values = wp_list_pluck( $columns, 'label' );

		if ( $part['allow_multiple_selection'] ) {
			// Checkbox mode
			foreach( $validated_value as $row_id => $row ) {
				$intersection = array_intersect( $valid_values, $row );

				if ( count( $row ) !== count( $intersection ) ) {
					return new WP_Error( 'error', __( 'Checkbox values are not valid.', 'happyforms' ) );
				}
			}
		} else {
			// Radio mode
			foreach( $validated_value as $value ) {
				if ( ! in_array( $value, $valid_values ) ) {
					return new WP_Error( 'error', __( 'Radio values are not valid.', 'happyforms' ) );
				}
			}
		}

		return $validated_value;
	}

	public function html_part_class( $class, $part, $form ) {
		if ( $this->type === $part['type'] ) {
			if ( $part['allow_multiple_selection'] ) {
				$class[] = 'happyforms-selection--multiple';
			}
		}

		return $class;
	}

	public function get_part_value( $value, $part, $form, $component ) {
		if ( $this->type === $part['type'] ) {
			if ( false !== $component ) {
				$value = isset( $value[$component] ) ? $value[$component] : '';
			}
		}

		return $value;
	}

	public function stringify_value( $value, $part, $form ) {
		if ( $this->type === $part['type'] ) {
			$rows = happyforms_get_part_options( $part['rows'], $part, $form );
			$lines = array();

			if ( $part['allow_multiple_selection'] ) {
				foreach( $rows as $row ) {
					$label = $row['label'];
					$row_id = $row['id'];
					$row_value = implode( ', ', $value[$row_id] );
					$lines[] = "{$label}: {$row_value}";
				}
			} else {
				foreach( $rows as $row ) {
					$label = $row['label'];
					$row_id = $row['id'];
					$row_value = $value[$row_id];
					$lines[] = "{$label}: {$row_value}";
				}
			}

			$value = implode( '<br>', $lines );
		}

		return $value;
	}

	public function message_part_value( $value, $original_value, $part, $destination ) {
		if ( isset( $part['type'] )
			&& $this->type === $part['type']
			&& 'email' === $destination ) {

			$value = "<br>{$value}";
		}

		return $value;
	}

	public function script_dependencies( $deps, $forms ) {
		$contains_table = false;
		$form_controller = happyforms_get_form_controller();

		foreach ( $forms as $form ) {
			if ( $form_controller->get_first_part_by_type( $form, $this->type ) ) {
				$contains_table = true;
				break;
			}
		}

		if ( ! happyforms_is_preview() && ! $contains_table ) {
			return $deps;
		}

		wp_register_script(
			'happyforms-part-table',
			happyforms_get_plugin_url() . 'assets/js/frontend/table.js',
			array(), HAPPYFORMS_VERSION, true
		);

		$deps[] = 'happyforms-part-table';

		return $deps;
	}

}
